@@ -6,71 +6,12 @@ use Mojo::Base 'Mojo::EventEmitter';
 use WWW::Crawler::Mojo::Job;
 use WWW::Crawler::Mojo::Queue::Memory;
 use WWW::Crawler::Mojo::UserAgent;
-use WWW::Crawler::Mojo::ScraperUtil qw{resolve_href decoded_body};
+use WWW::Crawler::Mojo::ScraperUtil qw{
+            collect_urls_css html_handlers resolve_href decoded_body};
 use Mojo::Message::Request;
-use Mojo::Util qw{xml_escape dumper};
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 has clock_speed => 0.25;
-has element_handlers => sub { {
-    'script[src]'   => sub { $_[0]->{src} },
-    'link[href]'    => sub { $_[0]->{href} },
-    'a[href]'       => sub { $_[0]->{href} },
-    'img[src]'      => sub { $_[0]->{src} },
-    'area'          => sub { $_[0]->{href}, $_[0]->{ping} },
-    'embed[src]'    => sub { $_[0]->{src} },
-    'frame[src]'    => sub { $_[0]->{src} },
-    'iframe[src]'   => sub { $_[0]->{src} },
-    'input[src]'    => sub { $_[0]->{src} },
-    'object[data]'  => sub { $_[0]->{data} },
-    'form'          => sub {
-        my $dom = shift;
-        my (%seed, $submit);
-        
-        $dom->find("[name]")->each(sub {
-            my $e = shift;
-            $seed{my $name = $e->{name}} ||= [];
-            
-            if ($e->type eq 'select') {
-                $e->find('option[selected]')->each(sub {
-                    push(@{$seed{$name}}, shift->{value});
-                });
-            } elsif ($e->type eq 'textarea') {
-                push(@{$seed{$name}}, $e->text);
-            }
-            
-            return unless (my $type = $e->{type});
-            
-            if (!$submit && grep{$_ eq $type} qw{submit image}) {
-                $submit = 1;
-                push(@{$seed{$name}}, $e->{value});
-            } elsif (grep {$_ eq $type} qw{text hidden number}) {
-                push(@{$seed{$name}}, $e->{value});
-            } elsif (grep {$_ eq $type} qw{checkbox}) {
-                push(@{$seed{$name}}, $e->{value}) if (exists $e->{checked});
-            } elsif (grep {$_ eq $type} qw{radio}) {
-                push(@{$seed{$name}}, $e->{value}) if (exists $e->{checked});
-            }
-        });
-        
-        return [$dom->{action} || '',
-                    uc ($dom->{method} || 'GET'), Mojo::Parameters->new(%seed)];
-    },
-    'meta[content]' => sub {
-        return $1 if ($_[0] =~ qr{http\-equiv="?Refresh"?}i &&
-                                (($_[0]->{content} || '') =~ qr{URL=(.+)}i)[0]);
-    },
-    'style' => sub {
-        my $dom = shift;
-        return collect_urls_css($dom->content);
-    },
-    '[style]' => sub {
-        collect_urls_css(shift->{style});
-    },
-    'urlset[xmlns^=http://www.sitemaps.org/schemas/sitemap/]' => sub {
-        @{$_->find('url loc')->map(sub {$_->content})->to_array};
-    }
-} };
 has max_conn => 1;
 has max_conn_per_host => 1;
 has queue => sub { WWW::Crawler::Mojo::Queue::Memory->new };
@@ -98,8 +39,7 @@ sub init {
                                         unless $self->has_subscribers('empty');
     $self->on('error', sub { say "An error occured during crawling $_[0]: $_[1]" })
                                         unless $self->has_subscribers('error');
-    $self->on('res', sub { $_[1]->() })
-                                        unless $self->has_subscribers('res');
+    $self->on('res', sub { $_[1]->() }) unless $self->has_subscribers('res');
     
     $self->ua->transactor->name($self->ua_name);
     $self->ua->max_redirects(5);
@@ -117,17 +57,18 @@ sub init {
 
 sub process_job {
     my $self = shift;
+    my $queue = $self->queue;
     
-    if (!$self->queue->length) {
+    if (!$queue->length) {
         $self->emit('empty') if (!$self->ua->active_conn);
         return;
     }
     if ($self->ua->active_conn >= $self->max_conn ||
-        $self->ua->active_host($self->queue->next->url) >= $self->max_conn_per_host) {
+        $self->ua->active_host($queue->next->url) >= $self->max_conn_per_host) {
         return;
     }
     
-    my $job = $self->queue->dequeue;
+    my $job = $queue->dequeue;
     my $url = $job->url;
     my $ua = $self->ua;
     my $tx = $ua->build_tx($job->method || 'get' => $url => $job->tx_params);
@@ -145,9 +86,8 @@ sub process_job {
             return;
         }
         
-        $self->emit('res', sub {
-            $self->scrape($res, $job, $_[0]);
-        }, $job, $res);
+        $self->emit('res',
+                        sub { $self->scrape($res, $job, $_[0]) }, $job, $res);
         
         $job->close;
     });
@@ -178,11 +118,11 @@ sub scrape {
             $base = resolve_href($base, $base_tag->attr('href'));
         }
         my $dom = Mojo::DOM->new(decoded_body($res));
-        for my $selector (sort keys %{$self->element_handlers}) {
+        state $handlers = html_handlers();
+        for my $selector (sort keys %{$handlers}) {
             $dom->find($selector)->each(sub {
                 my $dom = shift;
-                return if ($dom->xml && _wrong_dom_detection($dom));
-                for ($self->element_handlers->{$selector}->($dom)) {
+                for ($handlers->{$selector}->($dom)) {
                     $self->_delegate_enqueue($_, $dom, $job, $base, $cb);
                 }
             });
@@ -196,12 +136,8 @@ sub scrape {
     }
 };
 
-sub collect_urls_css {
-    map { s/^(['"])// && s/$1$//; $_ } (shift || '') =~ m{url\((.+?)\)}ig;
-}
-
 sub _delegate_enqueue {
-    my ($self, $url, $dom, $job, $base, $cb) = @_;
+    my ($self, $url, $context, $job, $base, $cb) = @_;
     my $method, my $params;
     
     return unless $url;
@@ -224,7 +160,7 @@ sub _delegate_enqueue {
     }
     
     $cb ||= sub { $_[1]->() };
-    $cb->($self, sub { $self->enqueue($_[0] || $child) }, $child, $dom);
+    $cb->($self, sub { $self->enqueue($_[0] || $child) }, $child, $context);
 }
 
 sub enqueue {
@@ -245,14 +181,6 @@ sub _urls_redirect {
     return @urls;
 }
 
-sub _wrong_dom_detection {
-    my $dom = shift;
-    while ($dom = $dom->parent) {
-        return 1 if ($dom->type && $dom->type eq 'script');
-    }
-    return;
-}
-
 1;
 
 =head1 NAME
@@ -270,13 +198,11 @@ WWW::Crawler::Mojo - A web crawling framework for Perl
     $bot->on(res => sub {
         my ($bot, $scrape, $job, $res) = @_;
         
-        $scrape->();
-    });
-    
-    $bot->on(refer => sub {
-        my ($bot, $enqueue, $job, $context) = @_;
-        
-        $enqueue->();
+        $scrape->(sub {
+            my ($bot, $enqueue, $job2, $context) = @_;
+            
+            $enqueue->();
+        });
     });
     
     $bot->enqueue('http://example.com/');
@@ -295,16 +221,6 @@ moderate range of web pages so DO NOT use it for persistent crawler jobs.
 L<WWW::Crawler::Mojo> inherits all attributes from L<Mojo::EventEmitter> and
 implements the following new ones.
 
-=head2 element_handlers
-
-HTML element handler on scraping.
-
-    my $handlers = $bot->element_handlers;
-    $bot->element_handlers->{img} = sub {
-        my $dom = shift;
-        return $dom->{src};
-    };
-
 =head2 clock_speed
 
 A number of millisecond for main event loop interval. Defaults to 0.25.
@@ -527,12 +443,6 @@ Collects URLs out of HTML.
         my ($uri, $dom) = @_;
     });
 
-=head2 collect_urls_css
-
-Collects URLs out of CSS.
-
-    @urls = collect_urls_css($dom);
-
 =head1 EXAMPLE
 
 L<https://github.com/jamadam/WWW-Flatten>
